#ifndef   ABOUTWINDOW_H
#include "AboutWindow.h"
#endif

/*
	Balance plugin for SoundPlay.
	(C) 2000, Tim Vernum <tpv@users.sourceforge.net>
	May be used freely in any SoundPlay plugin
*/

#ifndef   XBITMAP_H
#include "XBitmap.h"
#endif

#ifndef   BENAPHORE_H
#include "Benaphore.h"
#endif

#ifndef   AUTOBENAPHORE_H
#include "AutoBenaphore.h"
#endif

#ifndef   THREADOBJECT_H
#include "ThreadObject.h"
#endif

#include <Window.h>
#include <View.h>
#include <Bitmap.h>
#include <Autolock.h>

#include <stdio.h>

namespace SoundPlay
{
	namespace Plugins
	{
		namespace Balance
		{
//#pragma mark MainAboutRenderer
			class MainAboutRenderer : public Threads::ThreadObject
			{
				typedef Threads::ThreadObject inherited ;
				public:
					MainAboutRenderer( BBitmap * bmp , BView * v, Benaphore &lk ) ;
					int32 PerformThreadTask(void);
					void Stop();

				private:
					void ShiftColour( rgb_color& ) ;
					void CheckDirection( uint8& col , int8& dir) ;
					
					BBitmap * draw_bitmap ;
					BView * view ;
					Benaphore & lock ;

					bool keep_running , is_running ;
					
					BPoint pos ;
					int8 delta_x ;
					rgb_color delta_col ;
					struct
					{
						int8 red ;
						int8 blue ;
						int8 green ;
					} dir_col ;

					rgb_color new_colour ;
			} ;				

//#pragma mark MainAboutView
			class MainAboutView : public BView
			{
				typedef BView inherited ;
				public:
					MainAboutView( BRect r , uint32 fm , BBitmap *bmp ) ;
					~MainAboutView( void ) ;
					
					void DetachedFromWindow( void ) ;
					void Draw( BRect r ) ;
					
				private:
					BBitmap * draw_bitmap ;
					Benaphore lock  ;
					MainAboutRenderer renderer ;
			} ;
			
//#pragma mark LowerAboutView
			class LowerAboutView : public BView
			{
				typedef BView inherited ;
				public:
					LowerAboutView( BRect r, uint32 fm ) ;
					~LowerAboutView( void ) ;
					
					void Draw( BRect r ) ;
					void Pulse( void ) ;
					
				private:

					void LowerAboutView::SetNextStage( void ) ;
					bool LowerAboutView::AtNextStage( void ) ;

					// 0 = Balance v0.1
					// 1 = A SoundPlay Plugin
					// 2 = Copyright 2000
					// 3 = By Tim Vernum
					// 4 = <tpv@users.sourceforge.net>
					// 5 = . . . . .
					enum {
						nStrings = 6 ,
					} ;
					const char * strings[nStrings] ;
					uint32 currentString ;
					
					enum EStage
					{
						StageNone ,
						In_1 ,
						In_2 ,
						In_3 ,
						In_4 ,
						In_5 ,
						In_6 ,
						In_7 ,
						In_8 ,
						Out_8 ,
						Out_7 ,
						Out_6 ,
						Out_5 ,
						Out_4 ,
						Out_3 ,
						Out_2 ,
						Out_1 ,
					} stage ;
					
					bigtime_t nextStageAt ;
			} ;
			
//#pragma mark -
			AboutWindow::AboutWindow( void )
			:	inherited(
					BRect( 50,50, 400, 170 ) ,
					"About" ,
					B_FLOATING_WINDOW_LOOK ,
					B_NORMAL_WINDOW_FEEL ,
					B_NOT_ZOOMABLE | B_ASYNCHRONOUS_CONTROLS | B_NOT_RESIZABLE ) 
			{
				BRect r( Bounds() ) ;
				r.bottom -= 35 ;
				
				BBitmap * bitmap = new BBitmap( r, B_RGB32, true ) ;
				BView * drawer = new BView( r, "drawer", B_FOLLOW_ALL_SIDES, B_WILL_DRAW ) ;
				
				bitmap->AddChild( drawer ) ;

				bitmap->Lock() ;				
				BFont font( *be_bold_font ) ;
				
				font.SetFace( B_ITALIC_FACE | font.Face() ) ;

				float size = 80 ;
				int8 count = 0 ;
				while( true )
				{
					font.SetSize( size ) ;
					drawer->SetFont( &font ) ;
					float width = drawer->StringWidth( "Balance" ) ;
					if( width > drawer->Bounds().Width() )
					{
						size *= 4/5.0 ;
					}
					else if( width < drawer->Bounds().Width() * 2/3 )
					{
						size *= 6/5.0 ;
					}
					else
						break ;
						
					count++ ;
					if( count > 10 )
						break ;
				} 
				
				
				drawer->SetLowColor(0,0,0) ;
				drawer->FillRect( drawer->Bounds(), B_SOLID_LOW ) ;
				drawer->SetHighColor(255,255,255) ;
				BPoint pt = drawer->Bounds().LeftBottom() ;
				pt.x += 15 ;
				pt.y -= 10 ;
				drawer->DrawString( "Balance", pt ) ;
				
				drawer->Sync() ;
				bitmap->Unlock() ;				
				
				BView * child = new MainAboutView( r , B_FOLLOW_ALL_SIDES , bitmap ) ;
				AddChild( child ) ;
				
				r.top = r.bottom+1 ;
				r.bottom = Bounds().bottom ;
				
				child = new LowerAboutView( r, B_FOLLOW_LEFT_RIGHT | B_FOLLOW_BOTTOM ) ;
				AddChild( child ) ;
			}
			
//#pragma mark -
			MainAboutView:: MainAboutView( BRect r , uint32 fm , BBitmap * bmp )
			:	inherited( r, "MainAboutView", fm, B_WILL_DRAW )
			,	draw_bitmap( bmp )
			,	lock( "about_lock" )
			,	renderer( draw_bitmap, this, lock )
			{
				SetViewColor( B_TRANSPARENT_COLOR ) ;
				renderer.StartThread() ;
			}
			
			MainAboutView::~MainAboutView()
			{
				delete draw_bitmap ;
			}
			
			void MainAboutView::DetachedFromWindow(void)
			{
				renderer.Stop() ;
			}
			
			void MainAboutView:: Draw( BRect r )
			{
				AutoBenaphore autolock( lock ) ;
				this->DrawBitmapAsync( draw_bitmap , draw_bitmap->Bounds() , this->Bounds() ) ;
				this->Flush( ) ;
				this->Sync( ) ;
			}
			
//#pragma mark -
			MainAboutRenderer::MainAboutRenderer( BBitmap * bmp , BView * v, Benaphore& lk )
			:	inherited( "MainAboutRenderer" )
			,	draw_bitmap( bmp )
			,	view(v)
			,	lock( lk )
			,	keep_running(true)
			,	is_running(false)
			,	pos(0,0)
			,	delta_x(1)
			{
				new_colour.red = new_colour.blue = 225 ;
				new_colour.green = 255 ;
				new_colour.alpha = 255 ;

				delta_col.red = 225 ;
				delta_col.blue = 240 ;
				delta_col.green = 255 ;

				dir_col.red = -1 ;
				dir_col.blue = -1 ;
				dir_col.green = -1 ;

			}

			void MainAboutRenderer::ShiftColour(rgb_color& rgb)
			{
				rgb.red = static_cast<uint8>( static_cast<float>(rgb.red * delta_col.red)/255.0 ) ;
				rgb.blue = static_cast<uint8>( static_cast<float>(rgb.blue * delta_col.blue)/255.0) ;
				rgb.green = static_cast<uint8>( static_cast<float>(rgb.green * delta_col.green)/255.0) ;
			}
			
			void MainAboutRenderer::CheckDirection( uint8& col , int8& dir)
			{
				if( col < 5 )
				{
					dir = 1 ;
				}
				else if( col > 200 )
				{
					dir = -1 ;
				}
			}

			int32 MainAboutRenderer::PerformThreadTask( void )
			{
				is_running = true ;

				BBitmap ref_bitmap( draw_bitmap->Bounds() , draw_bitmap->ColorSpace(), false ) ;
				memcpy( ref_bitmap.Bits(), draw_bitmap->Bits(), ref_bitmap.BitsLength() ) ;
				
				while( keep_running )
				{
					// Bitmap locked
					{
						AutoBenaphore autolock( lock ) ;
						
						rgb_color rgb = Extensions::Bitmap::GetColourAt( &ref_bitmap, pos ) ;
						if( rgb.red || rgb.blue || rgb.green )
						{
							ShiftColour( rgb ) ;
							Extensions::Bitmap::SetColourAt( draw_bitmap, pos, rgb ) ;
						}	
					}
					// unlocked

					if( view->Window()->LockWithTimeout( 100000 ) == B_OK )
					{ // Window locked
						
						BRect r = view->Bounds()  ;
						float width  = r.Width()  ;
						float height = r.Height() ;

						r.left   = floor( pos.x * width  / draw_bitmap->Bounds().Width()  ) ;
						r.right  = ceil ( pos.x * width  / draw_bitmap->Bounds().Width()  ) ;

						r.top    = floor( pos.y * height / draw_bitmap->Bounds().Height() ) ;
						r.bottom = ceil ( pos.y * height / draw_bitmap->Bounds().Height() ) ;

						view->Invalidate( r ) ;
				
						view->Window()->Unlock() ;
					} // Window unlocked
					
					pos.y = pos.y + 1 ;
					if( pos.y > draw_bitmap->Bounds().bottom )
					{
						pos.y = 0 ;
						pos.x = pos.x + delta_x ;
						if( pos.x > draw_bitmap->Bounds().right || pos.x < 0 )
						{
							pos.x = pos.x - delta_x ;
							delta_x = -delta_x ;
						}
						delta_col.red += (dir_col.red*3) ;
						delta_col.blue += (dir_col.blue*2) ;
						delta_col.green += (dir_col.green*1) ;

#define CHECK_DIRECTION( _col_ ) this->CheckDirection( delta_col . _col_ , dir_col . _col_ ) ;
 						
						CHECK_DIRECTION( red ) ;
						CHECK_DIRECTION( blue ) ;
						CHECK_DIRECTION( green ) ;

						snooze( 250 ) ;
					}
				}

				is_running = false ;

				return 0 ;
			}
			
			void MainAboutRenderer::Stop()
			{
				printf( "Renderer Stopping..." ) ;
				keep_running = false ;
				while( is_running )
					snooze( 1000 ) ;
				printf( "...Stopped\n" ) ;
			}
			
			inline void LowerAboutView::SetNextStage( void )
			{
				nextStageAt = system_time() + 100000 ;
			}
			inline bool LowerAboutView::AtNextStage( void )
			{
				return system_time() >= nextStageAt ;
			}

			LowerAboutView::LowerAboutView( BRect r, uint32 fm )
			:	inherited( r, "LowerAboutView", fm, B_WILL_DRAW | B_PULSE_NEEDED )
			,	currentString(0)
			,	stage(StageNone)
			,	nextStageAt(0)
			{
				strings[0] = "Balance v0.1" ;
				strings[1] = "A SoundPlay plugin" ;
				strings[2] = "Copyright 2000" ;
				strings[3] = "By Tim Vernum" ;
				strings[4] = "tpv@users.sourceforge.net" ;
				strings[5] = ". . . . ." ;

				SetLowColor( 0,0,0,255 ) ;
				SetViewColor( 0,0,0,255 ) ;
				SetDrawingMode( B_OP_COPY ) ;

				SetNextStage() ;
			}
			
			LowerAboutView::~LowerAboutView( void )
			{
			}
			
			void LowerAboutView::Pulse( void )
			{
				if( AtNextStage() )
				{
					if( stage == Out_1 )
					{
						stage = StageNone ;
						currentString++ ;
						if( currentString >= nStrings )
							currentString = 0 ;
					}
					else
					{
						stage = static_cast<EStage>( static_cast<int>(stage) + 1 ) ;
					}
					Invalidate( Bounds() ) ;
					SetNextStage() ;
				}
			}
			
			void LowerAboutView::Draw( BRect )
			{
				if( stage == StageNone )
					return ; 

				float sw = StringWidth( strings[currentString] ) ;
				
				font_height fh ;
				GetFontHeight( &fh ) ;
				
				BRect r = Bounds() ;
				BPoint centre( r.Width()/2 , r.Height()/2 ) ;
				
				BPoint at( centre.x -sw/2 , centre.y + fh.descent + fh.ascent/2 ) ;
				
				switch( stage )
				{
					case StageNone:
					{
						// not needed
					}
					break ;
					
					case In_1:
					case Out_1:
					{
						SetHighColor( 64,160,64,255 ) ;
					}
					break ;

					case In_2:
					case Out_2:
					{
						SetHighColor( 64,160,160,255 ) ;
					}
					break ;

					case In_3:
					case Out_3:
					{
						SetHighColor( 64,64,160,255 ) ;
					}
					break ;
					
					case In_4:
					case Out_4:
					{
						SetHighColor( 160,64,160,255 ) ;
					}
					break ;
					
					case In_5:
					case Out_5:
					{
						SetHighColor( 160,64,64,255 ) ;
					}
					break ;

					case In_6:
					case Out_6:
					{
						SetHighColor( 160,160,64,255) ;
					}
					break ;

					case In_7:
					case Out_7:
					{
						SetHighColor( 160,160,160,255) ;
					}
					break ;
					
					case In_8:
					case Out_8:
					{
						SetHighColor( 255,255,255,255 ) ;
					}
					break ;
				}
				
				DrawString( strings[currentString], at ) ;
			}
		}
	}
}					


