
/*
	Balance plugin for SoundPlay.
	(C) 2000, Tim Vernum <tpv@users.sourceforge.net>
	May be used freely in any SoundPlay plugin
*/

#include <stdio.h>
#include <MediaDefs.h>
#include <View.h>

#include "pluginproto.h"

#ifndef   ABOUTWINDOW_H
#include "AboutWindow.h"
#endif

#ifndef   BALANCEWINDOW_H
#include "BalanceWindow.h"
#endif

#ifndef   FILTER_H
#include "Filter.h"
#endif

#ifndef   CONFIG_H
#include "Config.h"
#endif

#ifndef   CONFIGUREVIEW_H
#include "ConfigureView.h"
#endif

namespace SoundPlay
{
	namespace Plugins
	{
		namespace Balance
		{
			void	*	GetPlugin(void ** , const char * , const char * , uint32 , plugin_info * ) ;
			void		DestroyPlugin( void * ,void * ) ;
			void 		ShowAboutWindow( void ) ;

			BView *		ConfigureActive( void * ) ;
			BView *		ConfigureDefault( BMessage * ) ;

			plugin_descriptor plugin =
			{
				PLUGIN_DESCRIPTOR_MAGIC,
				PLUGIN_DESCRIPTOR_VERSION,
				"sp-balance",
				1,
				PLUGIN_IS_FILTER ,
			
				"Balance",
				"By Tim Vernum\n" "<tpv@users.sourceforge.net>\n\n",
				&ShowAboutWindow,    // about
				&ConfigureDefault,   // configure
				&GetPlugin,          // create
				&DestroyPlugin 		 // destroy
			} ;
			
			static plugin_descriptor * plugin_list[] =
			{
				// only 1 plugin
				&plugin,
				0
			} ;

			filter_plugin_ops filter_ops =
			{
				PLUGIN_FILTER_MAGIC,	// is a filter
				PLUGIN_FILTER_VERSION,	// of the latest version
				NULL,					// no filechange callback
				DoFilter,				// filter
				ConfigureActive,		// configure
				SetConfig,				// save
				GetConfig				// restore
			};
//#pragma mark -

			void *GetPlugin( void **data ,const char * , const char * , uint32, plugin_info *info )
			{
				BalanceWindow * win = new BalanceWindow ;
				*data=win;
			
				info->controller->AddWindow( win ) ;
				return &filter_ops;
			}
			
			void DestroyPlugin( void * , void *data )
			{
				BalanceWindow * win = reinterpret_cast<BalanceWindow *>( data ) ;
				win->Lock();
				win->Quit();
			}

//#pragma mark -

			void ShowAboutWindow( void )
			{
				(new AboutWindow)->Show() ; ;
			}
			
			BView * ConfigureActive( void * data )
			{
				BMessage * msg = new BMessage ;
				GetConfig( data, msg ) ;
				return new ConfigureView( data, msg, true ) ;
			}
			
			BView * ConfigureDefault( BMessage * msg ) 
			{
				return new ConfigureView( NULL, msg, false ) ;
			}
		}
	}
}			


//#pragma mark -
	
plugin_descriptor **get_plugin_list( void )
{
	return SoundPlay::Plugins::Balance::plugin_list ;
}
