/*
	Balance plugin for SoundPlay.
	(C) 2000, Tim Vernum <tpv@users.sourceforge.net>
	May be used freely in any SoundPlay plugin
*/

#ifndef   BALANCEWINDOW_H
#include "BalanceWindow.h"
#endif

#include "MSlider.h"
#include "MBorder.h"
#include "VGroup.h"

#include <stdio.h>

namespace SoundPlay
{
	namespace Plugins
	{
		namespace Balance
		{

			BalanceWindow::BalanceWindow()
			:	inherited (
						BRect(200,100,199+256,99+100) ,
						"Balance" ,
						B_TITLED_WINDOW ,
						B_ASYNCHRONOUS_CONTROLS )
			,	keepalive(true) 
			{
				SetupChildren(1) ;
			}

			MSlider * BalanceWindow::CreateSlider( const char * label, int min, int max )
			{
				MSlider * slider = new MSlider(
					label ,
					min ,
					max ,
					1 ,		
					NULL ,
					this ,
					B_TRIANGLE_THUMB
				) ;
						
				rgb_color left_col  = { 160, 224, 192, 255 } ; 
				rgb_color right_col = { 224, 160, 192, 255 } ;
				
				slider->SetBarColor( right_col ) ;
				slider->UseFillColor( true, &left_col ) ;
			
				
				slider->SetHashMarks( B_HASH_MARKS_BOTTOM ) ;
				slider->SetHashMarkCount( 9 ) ;
				
				return slider ;
			}
			
			void BalanceWindow::SetupChildren( int32 count )
			{
				if( count <= 1 )
				{
					slider1 = this->CreateSlider( "Balance", -100, 100 ) ;
					slider1->SetLimitLabels( "left", "right" ) ;
					
					slider2 = NULL ;
				}
				else
				{
					slider1 = this->CreateSlider( "Left", 0, 100 ) ;
					slider1->SetLimitLabels( "0", "10" ) ;

					slider2 = this->CreateSlider( "Right", 0, 100 ) ;
					slider2->SetLimitLabels( "0", "10" ) ;
				}
			
			
				this->AddChild
				(
					new MBorder
					(
						M_RAISED_BORDER,8,"border",
						new VGroup(
							slider1, slider2, NULL
						)
					)
				) ;
				
				Show();
			}
			
			BalanceWindow::~BalanceWindow()
			{
			}
			
			bool BalanceWindow::QuitRequested()
			{
				keepalive=false;
				return false;
			}
			
			status_t BalanceWindow::GetBalance( float& left , float& right )
			{
				if( !keepalive )
					return B_ERROR ;
				
				if( slider2 )
				{
					left  = slider1->Value()/50.0 ;
					right = slider2->Value()/50.0 ;
				}
				else
				{
					float slide_value = slider1->Value()/100.0 ;
					left = 1 - slide_value ;
					right = 1 + slide_value ;
				}				
// printf( "Balance = %f , %f\n", left, right ) ;
				return B_OK ;
			}
			
			status_t BalanceWindow::SetBalance( float left, float right )
			{
printf( "BalanceWindow::SetBalance: %f , %f\n", left, right ) ;
				if( slider2 )
				{
					slider1->SetValue(left*50) ;
					slider2->SetValue(right*50) ;
				}
				else
				{
					float slide_value ;
					if( left + right == 0 )
					{
						if( !left )
							slide_value = 0 ;
					}
					else
					{
						slide_value = (right-left) / (right+left) ;
					}
					slider1->SetValue( slide_value * 100.0 ) ;
				}
				return B_OK ;
			}
			
			int32 BalanceWindow::SliderCount( void )
			{
				return ( slider2 == NULL ? 1 : 2 ) ;
			}
			
			void BalanceWindow::SetSliderCount( int32 count )
			{
				if( count == SliderCount() )
					return ;
					
				float left,right ;
				GetBalance( left,right ) ;
				
				RemoveChild( ChildAt(0) ) ;
				SetupChildren( count ) ;
				
				SetBalance( left, right ) ;
			}
		}
	}
}
