#include "ConfigureView.h"

/*
	Balance plugin for SoundPlay.
	(C) 2000, Tim Vernum <tpv@users.sourceforge.net>
	May be used freely in any SoundPlay plugin
*/

#ifndef   CONFIG_H
#include "Config.h"
#endif

#include "VGroup.h"
#include "HGroup.h"
#include "MPopup.h"
#include "MCheckBox.h"
#include "MButton.h"
#include "Space.h"

#include "XMessage.h"

#include <Message.h>
#include <MenuItem.h>
#include <Window.h>

namespace SoundPlay
{
	namespace Plugins
	{
		namespace Balance
		{
			ConfigureView :: ConfigureView( void * _data, BMessage * _msg, bool _delMsg )
			:	inherited( M_NO_BORDER , 1 )
			,	data( _data )
			,	message( _msg )
			,	deleteMessage( _delMsg )
			,	dismissedOK(false)
			{
				AddChild( new VGroup( 
						sliderCount = new MPopup( "Number of Sliders", "1" , "2", NULL ) ,
						rebalance   = new MCheckBox( "Rebalance", 'RBal', false ) ,
						new HGroup(
							new Space( minimax(2,2,1E6,1E6,1) ) ,
							new MButton( "Cancel", new BMessage('Cncl'), this ) ,
							new MButton( "OK", new BMessage('Okay'), this ) ,
							NULL ) ,
						NULL ) ) ;
						
				int32 count ;
				if( message->FindInt32( "slider-count", &count ) == B_OK )
				{
					sliderCount->Menu()->ItemAt( count-1 )->SetMarked( true ) ;
				}
			}
			
			ConfigureView:: ~ConfigureView( void )
			{
				if( dismissedOK )
				{
					BMenu * menu = sliderCount->Menu() ;
					
					int32 count = 1 + ( menu->IndexOf( menu->FindMarked() ) ) ;
					
					Extensions::Message::SetInt32( message, "slider-count", count ) ;
					
					if( rebalance->Value() )
					{
						Extensions::Message::SetFloat( message, "left-balance", 1 ) ;
						Extensions::Message::SetFloat( message, "right-balance", 1 ) ;
					}
					
					if( data )
					{
						SetConfig( data, message ) ;
					}
				}
				
				if( deleteMessage )
				{
					delete message ;
				}
			}
			
			void ConfigureView::MessageReceived( BMessage * msg )
			{
				switch( msg->what )
				{
					case 'Okay' :
					{
						dismissedOK = true ;
					}
					// fall-through
					case 'Cncl' :
					{
						Window()->PostMessage( B_QUIT_REQUESTED ) ;
					}
					break ;
					
					default:
					{
						inherited::MessageReceived(msg) ;
					}
					break ;
				}
			}
		}		
	}
}
